@@ -41,7 +41,7 @@ import Foundation
 import _IOBluetooth
 import _lightbluecommon
 import _macutil
-import _BTUtil
+from _LightAquaBlue import BBServiceAdvertiser, BBBluetoothChannelDelegate
 
 import sets     # python 2.3
 
@@ -60,12 +60,14 @@ def _getavailableport(proto):
     # functionality for just getting an available channel.
 
     if proto == _lightbluecommon.RFCOMM:
-        result, channelID, servicerecordhandle = _BTUtil.BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(_BTUtil.BBServiceAdvertiser.serialPortProfileDictionary(), "DummyService", None)
+        try:
+            result, channelID, servicerecordhandle = BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(BBServiceAdvertiser.serialPortProfileDictionary(), "DummyService", None, None, None)
+        except:
+            result, channelID, servicerecordhandle = BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(BBServiceAdvertiser.serialPortProfileDictionary(), "DummyService", None)
         if result != _macutil.kIOReturnSuccess:
             raise _lightbluecommon.BluetoothError(result, \
                 "Could not retrieve an available service channel")
-        result = _BTUtil.BBServiceAdvertiser.removeService_(servicerecordhandle)
-        # what should I do if I can't remove the dummy service?
+        result = BBServiceAdvertiser.removeService_(servicerecordhandle)
         if result != _macutil.kIOReturnSuccess:
             raise _lightbluecommon.BluetoothError(result, \
                 "Could not retrieve an available service channel")
@@ -101,10 +103,6 @@ def _checkaddrpair(address, checkbtaddr=True):
             type(address[1]))              
 
 
-def _getsocketobject(proto):
-    return _SocketWrapper(_BluetoothSocket())
-
-
 # from std lib socket module
 class _closedsocket(object):
     __slots__ = []
@@ -212,14 +210,11 @@ class _SocketWrapper(object):
     dup.__doc__ = _lightbluecommon._socketdocs["dup"]
 
     def close(self):        
-        # call the 'close' method of the underlying _sock first
-        self._sock.close()       
+        self._sock.close()
         
-        # close the socket
         self._sock = _closedsocket()
         self.send = self.recv = self.sendto = self.recvfrom = self._sock._dummy
         
-        # try to stop advertising
         try:
             import lightblue
             lightblue.stopadvertise(self)
@@ -235,7 +230,7 @@ class _SocketWrapper(object):
     # delegate all other method calls to internal sock obj
     def __getattr__(self, attr):
         return getattr(self._sock, attr)
-        
+
 
 # internal _sock object for RFCOMM and L2CAP sockets
 class _BluetoothSocket(object):
@@ -243,9 +238,8 @@ class _BluetoothSocket(object):
     _boundports = { _lightbluecommon.L2CAP: sets.Set(),
                     _lightbluecommon.RFCOMM: sets.Set() }
 
-    # channel is the associated IOBluetoothRFCOMMChannel channel, if there is 
-    # one already (i.e. when a server socket spawns a new client socket)
-    def __init__(self, conn=None):    
+    # conn is the associated _RFCOMMConnection or _L2CAPConnection
+    def __init__(self, conn):    
         self.__conn = conn
         
         if conn is not None and conn.channel is not None:
@@ -256,7 +250,7 @@ class _BluetoothSocket(object):
         # timeout=None cos sockets default to blocking mode
         self.__timeout = None
         
-        self.__isserverspawned = (conn is not None)
+        #self.__isserverspawned = (conn.channel is not None)
         self.__port = 0
         self.__eventlistener = None
         self.__closed = False
@@ -275,16 +269,13 @@ class _BluetoothSocket(object):
         if not self.__islistening():
             raise _socket.error('Socket must be listening first')
         
-        # if queue of connected channels is empty, wait until a client connects
-        def nochannels():
-            return len(self.__queuedchannels) == 0
-        if nochannels():
-            self.__awaitavailable(nochannels, "accept timed out")
+        def clientconnected():
+            return len(self.__queuedchannels) > 0
+        if not clientconnected():
+            self.__waituntil(clientconnected, "accept timed out")
         
-        # get & remove first channel object from queue
         self.__queuedchannels_lock.acquire()
         try:
-            # get & remove first channel from queue
             newchannel = self.__queuedchannels.pop(0)
         finally:    
             self.__queuedchannels_lock.release()      
@@ -305,9 +296,9 @@ class _BluetoothSocket(object):
         if self.__conn.proto == _lightbluecommon.L2CAP:
             raise NotImplementedError("L2CAP server sockets not currently supported")
             
-        # if port==0, find an available port
-        if address[1] == 0:
-            address = (address[0], _getavailableport(self.__conn.proto))
+        if address[1] != 0:
+            raise _socket.error("must bind to port 0, other ports not supported on Mac OS X")
+        address = (address[0], _getavailableport(self.__conn.proto))
             
         # address must be either empty string or local device address
         if address[0] != "":
@@ -317,7 +308,6 @@ class _BluetoothSocket(object):
             except:
                 localaddr = None
             if localaddr is None or address[0] != localaddr:
-                # unknown host address
                 raise _socket.error(
                     errno.EADDRNOTAVAIL, os.strerror(errno.EADDRNOTAVAIL))            
                 
@@ -325,11 +315,11 @@ class _BluetoothSocket(object):
         if address[1] in self._boundports[self.__conn.proto]:
             raise _socket.error(errno.EADDRINUSE, os.strerror(errno.EADDRINUSE))
     
-        # note used port
         self._boundports[self.__conn.proto].add(address[1])
         self.__port = address[1]
         
     def close(self):
+        wasconnected = self.__isconnected() or self.__isbound()
         self.__stopevents()
         
         if self.__conn is not None:
@@ -346,7 +336,13 @@ class _BluetoothSocket(object):
         # process is talking to the device)
         if self.__remotedevice is not None: 
             self.__remotedevice.closeConnection()   # returns err code
-        
+            
+        # if you don't run the event loop a little here, it's likely you won't
+        # be able to reconnect to the same remote device later
+        if wasconnected:
+            _macutil.waitfor(0.5)
+
+            
     def connect(self, address):
         if self.__isbound():
             raise _socket.error("Can't connect, socket has been bound")
@@ -356,7 +352,7 @@ class _BluetoothSocket(object):
         
         # open a connection to device
         self.__remotedevice = _IOBluetooth.IOBluetoothDevice.withAddress_(
-                _macutil.btaddrtochars(address[0]))
+                _macutil.createbtdevaddr(address[0]))
         if not self.__remotedevice.isConnected():
             if self.__timeout is None:
                 result = self.__remotedevice.openConnection()
@@ -382,8 +378,16 @@ class _BluetoothSocket(object):
                 self.__eventlistener)   # pass listener as cocoa delegate
                 
         if result != _macutil.kIOReturnSuccess:
+            self.__remotedevice.closeConnection()
+            self.__stopevents()
+            self.__eventlistener = None
             raise _socket.error(result, 
                     "Cannot connect to %d on %s" % (address[1], address[0]))
+            return
+            
+        # if you don't run the event loop a little here, it's likely you won't
+        # be able to reconnect to the same remote device later
+        _macutil.waitfor(0.5)
 
     def connect_ex(self, address):
         try:
@@ -411,7 +415,8 @@ class _BluetoothSocket(object):
 
     def listen(self, backlog):
         if self.__islistening():
-            raise _socket.error("Socket is already listening")
+            return
+
         if not self.__isbound():
             raise _socket.error('Socket not bound')
         if not isinstance(backlog, int):
@@ -423,6 +428,16 @@ class _BluetoothSocket(object):
                     
         # start listening for client connections
         self.__startevents()
+        
+        
+    def _isclosed(self):
+        # isOpen() check doesn't work for incoming (server-spawned) channels            
+        if (self.__conn.proto == _lightbluecommon.RFCOMM and 
+                self.__conn.channel is not None and
+                not self.__conn.channel.isIncoming()):
+            return not self.__conn.channel.isOpen()    
+        return self.__closed
+
 
     def recv(self, bufsize, flags=0):
         if self.__commstate in (SHUT_RD, SHUT_RDWR):
@@ -436,26 +451,10 @@ class _BluetoothSocket(object):
         if bufsize == 0: 
             return ""
             
-        # TO DO: if the connection is closed, should this return empty string
-        # (like tcp sockets) or except (like pybluez) ? Check PyS60.
-        # - or is the exception only for hard resets, reboots etc.?
-
-        # maybe need this to ensure the isOpen() value below is up-to-date
+        # need this to ensure the _isclosed() check is up-to-date
         _macutil.looponce()
         
-        # TO DO: this doesn't work if self.__closed is true here and data 
-        # was sent by other side before they closed the conn. What happens is,
-        # the data they sent is not received because we get the closed event and
-        # miss all the data events that came before that. So this will think 
-        # the buffer is just empty, if a recv() hasn't been previously called
-        # to get the data events and put the received data in the buffer.
-        
-        # if other side has closed connection, just look at buffer
-        #if self.__closed or (self.__conn.proto == _lightbluecommon.RFCOMM and
-        #                     not self.__conn.channel.isOpen()):
-        if self.__closed:
-            #print "recv() called but closed", bufsize, len(self.__incomingdata)
-            #if self.__incomingdata.empty():
+        if self._isclosed():
             if len(self.__incomingdata) == 0:
                 raise _socket.error(errno.ECONNRESET,         
                                     os.strerror(errno.ECONNRESET))
@@ -463,17 +462,15 @@ class _BluetoothSocket(object):
     
         # if incoming data buffer is empty, wait until data is available or
         # channel is closed
-        def nodata():
-            return self.__incomingdata.empty() and not self.__closed
-        if nodata():
-            self.__awaitavailable(nodata, "recv timed out")
+        def gotdata():
+            return not self.__incomingdata.empty() or self._isclosed()
+        if not gotdata():
+            self.__waituntil(gotdata, "recv timed out")
 
         # other side closed connection while waiting?
-                #self.__incomingdata.empty():
-        if self.__closed and len(self.__incomingdata) == 0:    
+        if self._isclosed() and len(self.__incomingdata) == 0:    
             raise _socket.error(errno.ECONNRESET, os.strerror(errno.ECONNRESET))
             
-        # read in data up to buffer size
         return self.__incomingdata.read(bufsize)
         
         
@@ -511,28 +508,24 @@ class _BluetoothSocket(object):
                 # non-blocking with timeout
                 starttime = time.time()
         
-        # get channel MTU (a write can only contain up to the MTU size)
-        mtu = self.__conn.getwritemtu()
-        
         # loop until all data is sent
         writebuf = data
         bytesleft = len(data)
+        mtu = self.__conn.getwritemtu()
         while bytesleft > 0:
-            # if there's a timeout set, check whether operation has timed out
             if self.__timeout is not None and self.__timeout > 0:
                 if time.time() - starttime > self.__timeout:
                     raise _socket.timeout("send timed out")
-        
-            # calculate how many bytes I can send
-            if bytesleft > mtu: 
-                sendbytecount = mtu
-            else: 
-                sendbytecount = bytesleft
             
             # write the data to the channel (only the allowed amount)
             # the method/selector is the same for L2CAP and RFCOMM channels
-            result = self.__conn.channel.writeSync_length_(
-                    writebuf[:sendbytecount], sendbytecount)
+            if bytesleft > mtu: 
+                sendbytecount = mtu
+            else: 
+                sendbytecount = bytesleft            
+            #result = self.__conn.channel.writeSync_length_(
+            #        writebuf[:sendbytecount], sendbytecount)
+            result = self.__conn.write(writebuf[:sendbytecount])
 
             # normal tcp sockets don't seem to actually error on the first 
             # send() after a connection has broken; if you try a second time,
@@ -540,11 +533,9 @@ class _BluetoothSocket(object):
             if result != _macutil.kIOReturnSuccess:
                 raise _socket.error(result, "Error sending data")
             
-            # update bytesleft & data buffer
             bytesleft -= sendbytecount
             writebuf = writebuf[sendbytecount:] # remove the data just sent
             
-        # finished sending, return number of bytes sent
         return len(data) - bytesleft     
         
     # sendto args may be one of:
@@ -587,11 +578,9 @@ class _BluetoothSocket(object):
         
     def setblocking(self, flag):
         if flag == 0:
-            # set to non-blocking mode
-            self.__timeout = 0
+            self.__timeout = 0      # non-blocking
         else:
-            # set to blocking mode
-            self.__timeout = None
+            self.__timeout = None   # blocking
         
     def gettimeout(self):
         return self.__timeout
@@ -618,24 +607,28 @@ class _BluetoothSocket(object):
         if self.__isbound():
             return self.__port
         raise _lightbluecommon.BluetoothError("socket is neither connected nor bound")
+
+    # This method is called from outside this file.            
+    def _getchannel(self):
+        if self.__conn is None:
+            return None
+        return self.__conn.channel
         
     # Called by the event listener when data is available
+    # 'channel' is IOBluetoothRFCOMMChannel or IOBluetoothL2CAPChannel object
     def _handle_channeldata(self, channel, data):
-        """ 
-        Called when data is received on a channel. The <channel> is an
-        IOBluetoothRFCOMMChannel or IOBluetoothL2CAPChannel object, and <data>
-        is a string.
-        """
         self.__incomingdata.write(data)
+        _macutil.interruptwait()
 
     # Called by the event listener when a client connects to a server socket        
     def _handle_channelopened(self, channel):
         # put new channels into a queue, which 'accept' can then pull out
         self.__queuedchannels_lock.acquire()
         try:
-            # TODO implement max connections
+            # need to implement max connections
             #if len(self.__queuedchannels) < self.__maxqueuedconns:
             self.__queuedchannels.append(channel)
+            _macutil.interruptwait()
         finally:
             self.__queuedchannels_lock.release()
             
@@ -644,34 +637,35 @@ class _BluetoothSocket(object):
         # beware that this value won't actually be set until the event loop
         # has been driven so that this method is actually called
         self.__closed = True
+        _macutil.interruptwait()
             
-    def __awaitavailable(self, keepwaiting, timeoutmsg):
+    def __waituntil(self, stopwaiting, timeoutmsg):
         """
-        Waits until keepwaiting() returns false, or until the wait times out
+        Waits until stopwaiting() returns True, or until the wait times out
         (according to the self.__timeout value).
         
-        This is to make a function wait until a buffer has been filled.
-        i.e. keepwaiting() should return True if the buffer is still empty.
+        This is to make a function wait until a buffer has been filled. i.e.
+        stopwaiting() should return True when the buffer is no longer empty.
         """
-        if keepwaiting():
+        if not stopwaiting():
             if self.__timeout == 0:
                 # in non-blocking mode (immediate timeout)
                 # push event loop to really be sure there is no data available
                 _macutil.looponce()
-                if keepwaiting():
+                if not stopwaiting():
                     # trying to perform operation now would block
                     raise _socket.error(errno.EAGAIN, os.strerror(errno.EAGAIN))
             else:
                 # block and wait until we get data, or time out
-                if not _macutil.waitwhile(keepwaiting, self.__timeout):
-                    raise _socket.timeout(timeoutmsg)     
+                if not _macutil.waituntil(stopwaiting, self.__timeout):
+                    raise _socket.timeout(timeoutmsg)
                     
     def __createlistener(self):
         if self.__isbound():
-            return _ChannelServerEventListener.alloc().init(self, 
+            return _ChannelServerEventListener.alloc().initWithDelegate_port_protocol_(self, 
                     self._getport(), self.__conn.proto)
         else:
-            listener = _ChannelEventListener.alloc().init(self)
+            listener = _ChannelEventListener.alloc().initWithDelegate_(self)
             if self.__conn.channel is not None:
                 self.__conn.channel.setDelegate_(listener.delegate())
                 listener.registerclosenotif(self.__conn.channel)
@@ -727,8 +721,11 @@ class _RFCOMMConnection(object):
     def connect(self, device, port, listener):
         # open RFCOMM channel (should timeout actually apply to opening of
         # channel as well? if so need to do timeout with async callbacks)
-        result, self.channel = device.openRFCOMMChannelSync_withChannelID_delegate_(port, listener.delegate())
-        #result, self.channel = device.openRFCOMMChannel_channel_(port)
+        try:
+            # pyobjc 2.0
+            result, self.channel = device.openRFCOMMChannelSync_withChannelID_delegate_(None, port, listener.delegate())
+        except TypeError:
+            result, self.channel = device.openRFCOMMChannelSync_withChannelID_delegate_(port, listener.delegate())        
         if result == _macutil.kIOReturnSuccess:
             self.channel.setDelegate_(listener.delegate())
             listener.registerclosenotif(self.channel)
@@ -736,6 +733,13 @@ class _RFCOMMConnection(object):
             self.channel = None
         return result
         
+    def write(self, data):
+        if self.channel is None:
+            raise _socket.error("socket not connected")
+        return \
+            BBBluetoothChannelDelegate.synchronouslyWriteData_toRFCOMMChannel_(
+                buffer(data), self.channel)
+        
     def getwritemtu(self):
         return self.channel.getMTU()
         
@@ -752,7 +756,11 @@ class _L2CAPConnection(object):
         self.channel = channel    
 
     def connect(self, device, port, listener):
-        result, self.channel = device.openL2CAPChannelSync_withPSM_delegate_(port, listener.delegate())
+        try:
+            # pyobjc 2.0
+            result, self.channel = device.openL2CAPChannelSync_withPSM_delegate_(None, port, listener.delegate())
+        except TypeError:
+            result, self.channel = device.openL2CAPChannelSync_withPSM_delegate_(port, listener.delegate())            
         if result == _macutil.kIOReturnSuccess:
             self.channel.setDelegate_(listener.delegate())
             listener.registerclosenotif(self.channel)
@@ -760,6 +768,13 @@ class _L2CAPConnection(object):
             self.channel = None
         return result
         
+    def write(self, data):
+        if self.channel is None:
+            raise _socket.error("socket not connected")
+        return \
+            BBBluetoothChannelDelegate.synchronouslyWriteData_toL2CAPChannel_(
+                buffer(data), self.channel)        
+        
     def getwritemtu(self):
         return self.channel.getOutgoingMTU()        
 
@@ -775,7 +790,7 @@ class _ChannelEventListener(Foundation.NSObject):
     """
 
     # note this is a NSObject "init", not a python object "__init__"
-    def init(self, cb_obj):
+    def initWithDelegate_(self, cb_obj):
         """
         Arguments:
         - cb_obj: An object that receives callbacks when events occur. This
@@ -799,8 +814,10 @@ class _ChannelEventListener(Foundation.NSObject):
             raise TypeError("callback object is None")
         self.__cb_obj = cb_obj
         self.__closenotif = None
-        self.__channelDelegate = _BTUtil.BBBluetoothChannelDelegate.alloc().initWithDelegate_(self)
+        self.__channelDelegate = \
+                BBBluetoothChannelDelegate.alloc().initWithDelegate_(self)
         return self
+    initWithDelegate_ = objc.selector(initWithDelegate_, signature="@@:@")
         
     def delegate(self):
         return self.__channelDelegate
@@ -827,7 +844,7 @@ class _ChannelEventListener(Foundation.NSObject):
     # - (void)channelData:(id)channel data:(NSData *)data;
     def channelData_data_(self, channel, data):
         if hasattr(self.__cb_obj, '_handle_channeldata'):
-            self.__cb_obj._handle_channeldata(channel, str(data))
+            self.__cb_obj._handle_channeldata(channel, data[:])
     channelData_data_ = objc.selector(channelData_data_, signature="v@:@@")
 
     # implement method from BBBluetoothChannelDelegateObserver protocol:        
@@ -846,7 +863,7 @@ class _ChannelServerEventListener(Foundation.NSObject):
     """
     
     # note this is a NSObject "init", not a python object "__init__"
-    def init(self, cb_obj, port, proto):
+    def initWithDelegate_port_protocol_(self, cb_obj, port, proto):
         """
         Arguments:
         - cb_obj: to receive callbacks when a client connects to 
@@ -873,6 +890,8 @@ class _ChannelServerEventListener(Foundation.NSObject):
                 port)
         self.__usernotif = usernotif
         return self
+    initWithDelegate_port_protocol_ = objc.selector(
+        initWithDelegate_port_protocol_, signature="@@:@ii")
 
     def close(self):
         if self.__usernotif is not None:
@@ -891,7 +910,7 @@ class _ChannelServerEventListener(Foundation.NSObject):
                 self.__cb_obj._handle_channelopened(newChannel)
     # makes this method receive notif and channel as objects                
     newChannelOpened_channel_ = objc.selector(
-    		newChannelOpened_channel_, signature="v@:@@")
+            newChannelOpened_channel_, signature="v@:@@")
 
 
 # -----------------------------------------------------------
