@@ -29,7 +29,7 @@ import AppKit
 import objc
 
 import _IOBluetooth
-import _BTUtil
+import _LightAquaBlue
 import _lightbluecommon
 import _macutil
 import _bluetoothsockets
@@ -37,10 +37,10 @@ import _bluetoothsockets
 
 # public attributes
 __all__ = ("finddevices", "findservices", "finddevicename", 
+           "selectdevice", "selectservice",
            "gethostaddr", "gethostclass",
            "socket", 
-           "advertise", "stopadvertise", 
-           "selectdevice", "selectservice")
+           "advertise", "stopadvertise")
 
 # details of advertised services
 __advertised = {}
@@ -73,7 +73,7 @@ def findservices(addr=None, name=None, servicetype=None):
     services = []
     for devaddr in addresses:
         iobtdevice = _IOBluetooth.IOBluetoothDevice.withAddress_(
-            _macutil.btaddrtochars(devaddr))
+            _macutil.createbtdevaddr(devaddr))
             
         try:
             lastseen = iobtdevice.getLastServicesUpdate()
@@ -120,7 +120,7 @@ def finddevicename(address, usecache=True):
         return _gethostname()
 
     device = _IOBluetooth.IOBluetoothDevice.withAddress_(
-                _macutil.btaddrtochars(address))
+                _macutil.createbtdevaddr(address))
     if usecache:
         name = device.getName()
         if name is not None:
@@ -137,7 +137,7 @@ def finddevicename(address, usecache=True):
 ### local device ###
 
 def gethostaddr():
-    addr = _BTUtil.BBLocalDevice.getAddressString()
+    addr = _LightAquaBlue.BBLocalDevice.getAddressString()
     if addr is not None:
         # PyObjC returns all strings as unicode, but the address doesn't need 
         # to be unicode cos it's just hex values
@@ -146,14 +146,14 @@ def gethostaddr():
 
 
 def gethostclass():
-    cod = _BTUtil.BBLocalDevice.getClassOfDevice()
+    cod = _LightAquaBlue.BBLocalDevice.getClassOfDevice()
     if cod != -1:
         return int(cod)
     raise _lightbluecommon.BluetoothError("Cannot read local device class")                            
 
 
 def _gethostname():
-    name = _BTUtil.BBLocalDevice.getName()
+    name = _LightAquaBlue.BBLocalDevice.getName()
     if name is not None:
         return name
     raise _lightbluecommon.BluetoothError("Cannot read local device name")                            
@@ -177,15 +177,27 @@ def advertise(name, sock, servicetype):
     
     # advertise the service
     if servicetype == _lightbluecommon.RFCOMM:
-        result, finalchannelID, servicerecordhandle = _BTUtil.BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(
-            _BTUtil.BBServiceAdvertiser.serialPortProfileDictionary(), 
-            name, 
-            None)
+        try:
+            result, finalchannelID, servicerecordhandle = _LightAquaBlue.BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(
+                _LightAquaBlue.BBServiceAdvertiser.serialPortProfileDictionary(), 
+                name, 
+                None, None, None)
+        except:
+            result, finalchannelID, servicerecordhandle = _LightAquaBlue.BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(
+                _LightAquaBlue.BBServiceAdvertiser.serialPortProfileDictionary(), 
+                name, 
+                None)       
     elif servicetype == _lightbluecommon.OBEX:
-        result, finalchannelID, servicerecordhandle = _BTUtil.BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(
-            _BTUtil.BBServiceAdvertiser.objectPushProfileDictionary(), 
-            name, 
-            None)
+        try:
+            result, finalchannelID, servicerecordhandle = _LightAquaBlue.BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(
+                _LightAquaBlue.BBServiceAdvertiser.objectPushProfileDictionary(), 
+                name, 
+                None, None, None)
+        except:
+            result, finalchannelID, servicerecordhandle = _LightAquaBlue.BBServiceAdvertiser.addRFCOMMServiceDictionary_withName_UUID_channelID_serviceRecordHandle_(
+                _LightAquaBlue.BBServiceAdvertiser.objectPushProfileDictionary(), 
+                name, 
+                None)       
     else:
         raise ValueError("servicetype must be either RFCOMM or OBEX")
 
@@ -209,7 +221,7 @@ def stopadvertise(sock):
     if servicerecordhandle is None:
         raise _lightbluecommon.BluetoothError("no service advertised")
         
-    result = _BTUtil.BBServiceAdvertiser.removeService_(servicerecordhandle)
+    result = _LightAquaBlue.BBServiceAdvertiser.removeService_(servicerecordhandle)
     if result != _macutil.kIOReturnSuccess:
         raise _lightbluecommon.BluetoothError(
             result, "Error stopping advertising of service")
@@ -237,10 +249,10 @@ def selectdevice():
             devinfo = _getdevicetuple(results[0])
     
             # sometimes the baseband connection stays open which causes 
-            # problems with connections ... so close it here, see if this fixes 
+            # problems with connections w so close it here, see if this fixes 
             # it        
             dev = _IOBluetooth.IOBluetoothDevice.withAddress_(
-                        _macutil.btaddrtochars(devinfo[0]))        
+                        _macutil.createbtdevaddr(devinfo[0]))        
             if dev.isConnected(): 
                 dev.closeConnection()        
         
@@ -273,7 +285,7 @@ def selectservice():
             # problems with connections ... so close it here, see if this fixes 
             # it        
             dev = _IOBluetooth.IOBluetoothDevice.withAddress_(
-                        _macutil.btaddrtochars(serviceinfo[0]))        
+                        _macutil.createbtdevaddr(serviceinfo[0]))        
             if dev.isConnected(): 
                 dev.closeConnection()        
             
@@ -299,7 +311,8 @@ class _SDPQueryRunner(Foundation.NSObject):
         
         # performSDPQuery_ is async, so block-wait
         self._queryresult = None
-        if not _macutil.waitwhile(lambda: self._queryresult is None, timeout):
+        if not _macutil.waituntil(lambda: self._queryresult is not None,
+                                          timeout):
             raise _lightbluecommon.BluetoothError(
                 "Timed out getting services for %s" % \
                     device.getNameOrAddress())
@@ -311,6 +324,7 @@ class _SDPQueryRunner(Foundation.NSObject):
     def sdpQueryComplete_status_(self, device, status):
         # can't raise exception during a callback, so just keep the err value
         self._queryresult = status
+        _macutil.interruptwait()
     sdpQueryComplete_status_ = objc.selector(
         sdpQueryComplete_status_, signature="v@:@i")    # accept object, int
             
@@ -348,7 +362,7 @@ class _SyncDeviceInquiry(object):
         
         # wait until the inquiry is complete
         self._inquiring = True
-        _macutil.waitwhile(lambda: self._inquiring)
+        _macutil.waituntil(lambda: not self._inquiring)
         
         # if error occured during inquiry, raise exception
         if self._inquiryerr != _macutil.kIOReturnSuccess:
@@ -360,9 +374,10 @@ class _SyncDeviceInquiry(object):
         return [_getdevicetuple(device) for device in \
                     self._inquiry.getfounddevices()]
         
-    def _inquirycomplete(self, err, aborted):
+    def _inquirycomplete(self, err, aborted): 
         self._inquiryerr = err
         self._inquiring = False
+        _macutil.interruptwait()
         
         
 # Wrapper around IOBluetoothDeviceInquiry, with python callbacks that you can
@@ -443,7 +458,7 @@ class _AsyncDeviceInquiry(Foundation.NSObject):
         if self.cb_completed:
             self.cb_completed(err, aborted)
     deviceInquiryComplete_error_aborted_ = objc.selector(
-        deviceInquiryComplete_error_aborted_, signature="v@:@ic")
+        deviceInquiryComplete_error_aborted_, signature="v@:@iB")
              
     # - (void)deviceInquiryStarted:(IOBluetoothDeviceInquiry*)sender;             
     def deviceInquiryStarted_(self, inquiry):
@@ -503,9 +518,15 @@ def _getservicetuple(servicerecord):
     """
     addr = _macutil.formatdevaddr(servicerecord.getDevice().getAddressString())            
     name = servicerecord.getServiceName()
-    result, channel = servicerecord.getRFCOMMChannelID_()
+    try:
+        result, channel = servicerecord.getRFCOMMChannelID_(None) # pyobjc 2.0
+    except TypeError:
+        result, channel = servicerecord.getRFCOMMChannelID_()
     if result != _macutil.kIOReturnSuccess:
-        result, channel = servicerecord.getL2CAPPSM_()
+        try:
+            result, channel = servicerecord.getL2CAPPSM_(None) # pyobjc 2.0
+        except:
+            result, channel = servicerecord.getL2CAPPSM_()            
         if result != _macutil.kIOReturnSuccess:    
             channel = None
     return (addr, channel, name)    
