@@ -32,6 +32,11 @@
 #include <es_sock.h>
 #include <bt_sock.h>
 #include <bttypes.h>
+
+#if defined(__SYMBIAN_9__) || defined(__SYMBIAN_8__)
+#   include <bt_subscribe.h>
+#endif
+
 #include <btextnotifiers.h>     // TBTDeviceResponseParamsPckg
 #include <btdevice.h>
 
@@ -396,40 +401,47 @@ static PyObject* LightBlue_DiscoverDevices(PyObject* self, PyObject* args)
 // Returns an error code.
 static TInt GetLocalAddress(TBTDevAddr& aAddress)
 {
-#ifdef __HAS_BT_SUBSCRIBE__
-  TPtr8 ptr(aAddress.Des());
-  TInt err = RProperty::Get(KPropertyUidBluetoothCategory,
-			      KPropertyKeyBluetoothLocalDeviceAddress,
-			      ptr);
+    TInt err = KErrNone;
+
+#if defined(__SYMBIAN_9__)
+    TPtr8 ptr(aAddress.Des());
+    err = RProperty::Get(KPropertyUidBluetoothCategory,
+                         KPropertyKeyBluetoothGetLocalDeviceAddress,
+                         ptr);
+#elif defined(__SYMBIAN_8__)
+    TPtr8 ptr(aAddress.Des());
+    err = RProperty::Get(KPropertyUidBluetoothCategory,
+                         KPropertyKeyBluetoothLocalDeviceAddress,
+                         ptr);
 #else
-  RSocketServ socketServ;
-  TInt err = socketServ.Connect();
-  if (err)
-    return err;
-
-  // this solution comes from the "bthci" Series 60 example;
-  // does not work on Symbian 8-up
-  RSocket socket;
-  err = socket.Open(socketServ, KBTAddrFamily, KSockSeqPacket, KL2CAP);
-  if (!err) {
-    TPckgBuf<TBTDevAddr> btDevAddrPckg;
-    TRequestStatus status;
-    socket.Ioctl(KHCILocalAddressIoctl, status, &btDevAddrPckg, KSolBtHCI);
-    User::WaitForRequest(status);
-    err = status.Int();
+    RSocketServ socketServ;
+    err = socketServ.Connect();
+    if (err)
+        return err;
+    
+    // this solution comes from the "bthci" Series 60 example;
+    // does not work on Symbian 8-up
+    RSocket socket;
+    err = socket.Open(socketServ, KBTAddrFamily, KSockSeqPacket, KL2CAP);
     if (!err) {
-      TPtrC8 src(btDevAddrPckg().Des());
-      TPtr8 dest(aAddress.Des());
-      dest.Copy(src);
+        TPckgBuf<TBTDevAddr> btDevAddrPckg;
+        TRequestStatus status;
+        socket.Ioctl(KHCILocalAddressIoctl, status, &btDevAddrPckg, KSolBtHCI);
+        User::WaitForRequest(status);
+        err = status.Int();
+        if (!err) {
+            TPtrC8 src(btDevAddrPckg().Des());
+            TPtr8 dest(aAddress.Des());
+            dest.Copy(src);
+        }
+    
+        socket.Close();
     }
-
-    socket.Close();
-  }
-  
-  socketServ.Close();
+    
+    socketServ.Close();
 #endif
 
-  return err;
+    return err;
 }
 
 // Wraps GetLocalAddress() to provide a Python method interface.
@@ -513,60 +525,72 @@ static PyObject* LightBlue_GetLocalName(PyObject* self, PyObject* args)
 //  - aDeviceData - object to hold the retrieved class data.
 //
 // Returns an error code.
-static TInt GetLocalDeviceClass(TDeviceData& aDeviceData)
+//static TInt GetLocalDeviceClass(TDeviceData& aDeviceData)
+static TInt GetLocalDeviceClass(TBTDeviceClass& aDeviceClass)
 {
-    TInt err = KErrNone;    
-    
+    TInt err = KErrNone;
+
+#if defined(__SYMBIAN_9__)
+    TInt cod;
+    err = RProperty::Get(KPropertyUidBluetoothCategory,
+                  KPropertyKeyBluetoothGetDeviceClass,
+                  cod);
+    if (err == KErrNone) {
+        aDeviceClass = TBTDeviceClass(cod);
+    }
+#elif defined(__SYMBIAN_8__)
+    TInt cod;
+    err = RProperty::Get(KPropertyUidBluetoothCategory,
+                  KPropertyKeyBluetoothDeviceClass,
+                  cod);
+    if (err == KErrNone) {
+        aDeviceClass = TBTDeviceClass(cod);
+    }    
+#else
+
     RSocketServ socketServ;
     RSocket sock;
     
     err = socketServ.Connect();
     if (!err) {
         err = sock.Open(socketServ, KBTAddrFamily, KSockSeqPacket, KL2CAP);
-        
         if (!err) {
-            
             THCIDeviceClassBuf codBuf;
             TRequestStatus status;
             
             sock.Ioctl(KHCIReadDeviceClassIoctl, status, &codBuf, KSolBtHCI);
             User::WaitForRequest(status);
             
-			// THIS ERROR CHECK is new, so check it works
             if (status.Int() == KErrNone) {
-            
-				aDeviceData.iServiceClass = codBuf().iMajorServiceClass;
-				aDeviceData.iMajorClass = codBuf().iMajorDeviceClass;
-				aDeviceData.iMinorClass = codBuf().iMinorDeviceClass;
-				
-			}
-				  
+                aDeviceClass = TBTDeviceClass(codBuf().iMajorServiceClass,
+                                              codBuf().iMajorDeviceClass,
+                                              codBuf().iMinorDeviceClass);
+			}	  
             sock.Close();
         }
         socketServ.Close();
     }
+#endif
     return err;
 }
 
 // Wraps GetLocalDeviceClass() to provide a Python method interface.
 // Takes no arguments.
 //
-// Returns the local device class as a (service, major, minor) tuple.
+// Returns the local device class (an integer).
 static PyObject* LightBlue_GetLocalDeviceClass(PyObject* self, PyObject* args) 
 {
-    TDeviceData deviceData;
+    //TDeviceData aDeviceData;
+    TBTDeviceClass aDeviceClass;
     
     if (!PyArg_ParseTuple(args, ""))
         return NULL;
     
-    TInt err = GetLocalDeviceClass(deviceData);
+    TInt err = GetLocalDeviceClass(aDeviceClass);
     if (err) 
         return SPyErr_SetFromSymbianOSErr(err);
         
-    return Py_BuildValue("(iii)", 
-                         deviceData.iServiceClass,
-                         deviceData.iMajorClass,
-                         deviceData.iMinorClass);
+    return Py_BuildValue("i", aDeviceClass.DeviceClass());
 }
 
 
@@ -579,26 +603,24 @@ extern TInt apn_resolver_ConstructType();
 
 // define module methods
 static const PyMethodDef _lightblueutil_methods[] = {
-    {"lookupName", LightBlue_LookupName, METH_VARARGS},
-    {"selectDevice", LightBlue_SelectDevice, METH_VARARGS},    
-    {"discoverDevices", LightBlue_DiscoverDevices, METH_VARARGS},
-    {"getLocalAddress", LightBlue_GetLocalAddress, METH_VARARGS},
-    {"getLocalName", LightBlue_GetLocalName, METH_VARARGS},
-    {"getLocalDeviceClass", LightBlue_GetLocalDeviceClass, METH_VARARGS},
+    {"lookupName", (PyCFunction)LightBlue_LookupName, METH_VARARGS},
+    {"selectDevice", (PyCFunction)LightBlue_SelectDevice, METH_VARARGS},    
+    {"discoverDevices", (PyCFunction)LightBlue_DiscoverDevices, METH_VARARGS},
+    {"getLocalAddress", (PyCFunction)LightBlue_GetLocalAddress, METH_VARARGS},
+    {"getLocalName", (PyCFunction)LightBlue_GetLocalName, METH_VARARGS},
+    {"getLocalDeviceClass", (PyCFunction)LightBlue_GetLocalDeviceClass, METH_VARARGS},
     {"AoResolver", (PyCFunction)apn_resolver_new, METH_NOARGS},
     {0, 0}
 };
 
 
 /* initialise the module */
-extern "C" {
-    DL_EXPORT(void) init_lightblueutil(void)
-    {
-        /* Create the module and add the functions */
-        Py_InitModule("_lightblueutil", (PyMethodDef*)_lightblueutil_methods);
-        
-        if (apn_resolver_ConstructType() < 0) return;
-    }
+DL_EXPORT(void) init_lightblueutil(void)
+{
+    /* Create the module and add the functions */
+    Py_InitModule("_lightblueutil", (PyMethodDef*)_lightblueutil_methods);
+    
+    if (apn_resolver_ConstructType() < 0) return;
 }
 
 
